package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.bean.params.BasicExamStudentParam;
import com.qmth.distributed.print.business.bean.result.BasicExamStudentResult;
import com.qmth.distributed.print.business.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 考生字典表 服务类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
public interface BasicExamStudentService extends IService<BasicExamStudent> {

    /**
     * 分页查询
     *
     * @param requestUser     请求人
     * @param semesterId      学期id
     * @param examId          考试id
     * @param courseCode      课程代码
     * @param teacher         教师
     * @param college         学院
     * @param major           专业
     * @param teachClazzId      教学班id
     * @param examStudentInfo 考生信息（姓名/学号）
     * @param pageNumber      分页页码
     * @param pageSize        分页数量
     * @return 查询结果
     */
    IPage<BasicExamStudentResult> page(SysUser requestUser, Long semesterId, Long examId, String courseCode, String teacher,
            String college, String major, Long teachClazzId, String examStudentInfo, int pageNumber, int pageSize);

    /**
     * 新增编辑
     *
     * @param basicExamStudentParam 参数
     * @param requestUser           用户
     * @return id
     */
    Long edit(BasicExamStudentParam basicExamStudentParam, SysUser requestUser);

    /**
     * 导入
     *
     * @param file excel文件
     */
    void importLogic(MultipartFile file);

    /**
     * 导出
     *
     * @param response        response
     * @param schoolId        学校id
     * @param semesterId      学期id
     * @param examId          考试id
     * @param courseCode      课程代码
     * @param teacher         教师
     * @param college         学院
     * @param major           专业
     * @param teachClazz      教学班
     * @param examStudentInfo 考生信息（姓名/学号）
     */
    void exportLogic(HttpServletResponse response, Long schoolId, Long semesterId, Long examId, String courseCode,
            String teacher, String college, String major, String teachClazz, String examStudentInfo);

    /**
     * 批量删除
     *
     * @param idList 要删除的考生字典id集合
     */
    void deleteBatch(List<Long> idList);
}
