package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.entity.TBTaskPdf;
import com.qmth.distributed.print.business.mapper.TBTaskPdfMapper;
import com.qmth.distributed.print.business.service.TBTaskPdfService;
import com.qmth.teachcloud.common.bean.result.TbTaskDetailResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.enums.TaskTypeEnum;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 导入导出任务表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-02-06
 */
@Service
public class TBTaskPdfServiceImpl extends ServiceImpl<TBTaskPdfMapper, TBTaskPdf> implements TBTaskPdfService {

    @Override
    public boolean updateStatusById(TaskStatusEnum status, TaskResultEnum result, String summary, Long id) {
        UpdateWrapper<TBTaskPdf> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<TBTaskPdf> lambda = updateWrapper.lambda();
        lambda.set(TBTaskPdf::getStatus, status)
                .set(TBTaskPdf::getResult, result);
        if (StringUtils.isNotBlank(summary)) {
            lambda.set(TBTaskPdf::getSummary, status);
        }
        lambda.eq(TBTaskPdf::getId, id);
        return this.update(updateWrapper);
    }

    @Override
    public TBTaskPdf saveTask(TbTaskDetailResult tbTaskDetailResult, TaskTypeEnum createPdf, Long printPlanId, SysUser user, Long examDetailId) {
        try {
            TBTaskPdf tbTaskPdf = this.getById(examDetailId);
            if (tbTaskPdf == null) {
                tbTaskPdf = new TBTaskPdf(examDetailId, user.getSchoolId(), tbTaskDetailResult.getSemesterId(), tbTaskDetailResult.getExamId(), printPlanId, tbTaskDetailResult.getCourseNameCode(), tbTaskDetailResult.getPaperNumber(), user.getId());
                this.save(tbTaskPdf);
            }
            return tbTaskPdf;
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return null;
    }
}
