CREATE TABLE `course_target` (
    `id` BIGINT(20) NOT NULL COMMENT 'id',
    `school_id` BIGINT(20) NOT NULL COMMENT '学校id',
    `exam_id` BIGINT(20) NOT NULL COMMENT '考试id',
    `course_code` VARCHAR(20) NOT NULL COMMENT '课程编号',
    `target_name` VARCHAR(50) NULL COMMENT '课程目标',
    `target_content` MEDIUMTEXT NULL COMMENT '毕业要求指标',
    `degree_requirement` VARCHAR(255) NULL COMMENT '毕业要求',
    `total_weight` DECIMAL(6 , 2 ) NULL COMMENT '课程目标整体权重',
    `create_id` BIGINT(20) NULL COMMENT '创建人',
    `create_time` BIGINT(20) NULL COMMENT '创建时间',
    `update_id` BIGINT(20) NULL COMMENT '更新人',
    `update_time` BIGINT(20) NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
)  ENGINE=INNODB DEFAULT CHARACTER SET=UTF8MB4 COMMENT='课程目标表';

ALTER TABLE `course_target` 
ADD UNIQUE INDEX `course_target_unique` (`school_id` ASC,`exam_id` ASC, `course_code` ASC, `target_name` ASC) VISIBLE;
;

CREATE TABLE `course_evaluation` (
    `id` BIGINT(20) NOT NULL COMMENT 'id',
    `school_id` BIGINT(20) NOT NULL COMMENT '学校id',
    `exam_id` BIGINT(20) NOT NULL COMMENT '考试id',
    `course_code` VARCHAR(20) NOT NULL COMMENT '课程编号',
    `evaluation` VARCHAR(255) NOT NULL COMMENT '评价方式',
    `evaluation_desc` VARCHAR(255) NULL COMMENT '评价方式描述',
    `create_id` BIGINT(20) NULL COMMENT '创建人id',
    `create_time` BIGINT(20) NULL COMMENT '创建时间',
    `update_id` BIGINT(20) NULL COMMENT '更新人id',
    `update_time` BIGINT(20) NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
)  ENGINE=INNODB DEFAULT CHARACTER SET=UTF8MB4 COMMENT='课程评价方式表';

ALTER TABLE `course_evaluation` 
ADD UNIQUE INDEX `course_evaluation_unique` (`school_id` ASC, `exam_id` ASC, `course_code` ASC, `evaluation` ASC) VISIBLE;
;

CREATE TABLE `course_weight` (
  `id` BIGINT(20) NOT NULL COMMENT 'id',
  `school_id` BIGINT(20) NOT NULL COMMENT '学校id',
  `exam_id` BIGINT(20) NOT NULL COMMENT '考试id',
  `course_code` VARCHAR(20) NOT NULL COMMENT '课程编号',
  `target_id` BIGINT(20) NOT NULL COMMENT '课程目标表id',
  `evaluation_id` BIGINT(20) NOT NULL COMMENT '课程评价表id',
  `enable` TINYINT(1) NOT NULL DEFAULT '1' COMMENT '使用状态 1:使用 0:未使用',
  `weight` DECIMAL(6,2) NULL COMMENT '权重',
  `target_score` DECIMAL(6,2) NULL COMMENT '目标分值',
  `create_id` BIGINT(20) NULL COMMENT '创建人',
  `create_time` BIGINT(20) NULL COMMENT '创建时间',
  `update_id` BIGINT(20) NULL COMMENT '更新人',
  `update_time` BIGINT(20) NULL COMMENT '更新时间',
  PRIMARY KEY (`id`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8mb4
COMMENT = '课程权重设置表';

ALTER TABLE `course_weight` 
ADD UNIQUE INDEX `course_weight_unique` (`school_id` ASC, `exam_id` ASC, `course_code` ASC, `target_id` ASC, `evaluation_id` ASC) VISIBLE;
;

CREATE TABLE `basic_teach_clazz` (
  `id` BIGINT(20)NOT NULL COMMENT 'id',
  `school_id` BIGINT(20)NOT NULL COMMENT '学校id',
  `exam_id` BIGINT(20)NOT NULL COMMENT '考试id',
  `course_code` VARCHAR(20) NOT NULL COMMENT '课程编号',
  `clazz_name` VARCHAR(50) NOT NULL COMMENT '教学班名称',
  `create_id` BIGINT(20)NULL COMMENT '创建人',
  `create_time` BIGINT(20)NULL COMMENT '创建时间',
  `update_id` BIGINT(20)NULL COMMENT '更新人',
  `update_time` BIGINT(20)NULL COMMENT '更新时间',
  PRIMARY KEY (`id`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8mb4
COMMENT = '基础教学班表';

ALTER TABLE `basic_teach_clazz` 
ADD UNIQUE INDEX `basic_teach_clazz_unique` (`school_id` ASC, `exam_id` ASC, `clazz_name` ASC) VISIBLE;
;

CREATE TABLE `basic_exam_student` (
    `id` BIGINT(20) NOT NULL COMMENT 'id',
    `school_id` BIGINT(20) NOT NULL COMMENT '学校id',
    `semester_id` BIGINT(20) NOT NULL COMMENT '学期id',
    `exam_id` BIGINT(20) NOT NULL COMMENT '考试id',
    `course_code` VARCHAR(20) NOT NULL COMMENT '课程编号',
    `clazz_id` BIGINT(20) NOT NULL COMMENT '教学班id（basic_teach_clazz）',
    `paper_number` VARCHAR(30) DEFAULT NULL COMMENT '试卷编号',
    `student_name` VARCHAR(50) NOT NULL COMMENT '姓名',
    `student_code` VARCHAR(50) NOT NULL COMMENT '学号',
    `college` VARCHAR(50) NULL COMMENT '学院',
    `major` VARCHAR(50) NULL COMMENT '专业',
    `teacher_id` BIGINT(20) NULL COMMENT '任课教师id',
    `exam_start_time` BIGINT(20) DEFAULT NULL COMMENT '考试开始时间',
    `exam_end_time` BIGINT(20) DEFAULT NULL COMMENT '考试结束时间',
    `exam_place` VARCHAR(50) DEFAULT NULL COMMENT '考点',
    `exam_room` VARCHAR(50) DEFAULT NULL COMMENT '考场',
    `create_id` BIGINT(20) NULL COMMENT '创建人',
    `create_time` BIGINT(20) NULL COMMENT '创建时间',
    `update_id` BIGINT(20) NULL COMMENT '更新人',
    `update_time` BIGINT(20) NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
)  ENGINE=INNODB DEFAULT CHARACTER SET=UTF8MB4 COMMENT='考生字典表';

ALTER TABLE `basic_exam_student` 
ADD UNIQUE INDEX `basic_exam_student_unique` (`school_id` ASC, `exam_id` ASC, `course_code` ASC, `student_code` ASC) VISIBLE;
;


