package com.qmth.distributed.print.api;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.TCFinalScore;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.service.TCFinalScoreService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 期末考试成绩表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-成绩管理-期末考试成绩Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
@Validated
public class TCFinalScoreController {
    private final static Logger log = LoggerFactory.getLogger(TCFinalScoreController.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @Resource
    MarkQuestionService markQuestionService;

    @ApiOperation(value = "导入期末成绩-模板下载")
    @RequestMapping(value = "/final_score/template_download", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EXPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void finalScoreTemplateDownload(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                           @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                           @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        List<MarkQuestion> markQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndPaperType(examId, paperNumber, null);
        if (CollectionUtils.isEmpty(markQuestionList)) {
            throw ExceptionResultEnum.ERROR.exception("未找到试卷结构");
        }

        List<ExcelField> excelFieldList = new ArrayList<>();
        excelFieldList.add(new ExcelField("studentCode", "学号", true));
        excelFieldList.add(new ExcelField("name", "姓名", true));
        excelFieldList.add(new ExcelField("score", "成绩", true));

        for (int i = 0; i < markQuestionList.size(); i++) {
            MarkQuestion markQuestion = markQuestionList.get(i);
            excelFieldList.add(new ExcelField("key" + (i + 1), markQuestion.getMainNumber() + "-" + markQuestion.getSubNumber(), true));
        }
        printCommonService.scoreDownLoadExaminationTemplate("1、所有字段均为必填字段；\n" +
                "2、期末成绩按总分，和小题分录入，题号按大题号-小题号方式录入，如下，1-1表示第一大题第一小题，2-1表示第二大题第一小题；\n" +
                "3、请不要删除此行，也不要删除模板中的任何列；\n" +
                "4、使用前请先删除样例数据。", "sheet1", "期末成绩导入模版", excelFieldList);
    }

    @ApiOperation(value = "导入期末成绩")
    @RequestMapping(value = "/final_score/import", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.IMPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    public Object finalScoreExamImport(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                                       @ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                       @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                       @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        return ResultUtil.ok(tcFinalScoreService.finalScoreExcelImport(file, examId, courseCode, paperNumber));
    }

    @ApiOperation(value = "期末成绩列表")
    @RequestMapping(value = "/final_score/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = TCFinalScore.class)})
    public Object finalScoreList(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                 @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                 @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                 @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) throws IOException {
        return ResultUtil.ok(tcFinalScoreService.finalScoreList(new Page<>(pageNumber, pageSize), examId, courseCode, paperNumber));
    }

    @ApiOperation(value = "期末成绩编辑")
    @RequestMapping(value = "/final_score/edit", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EDIT)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = TCFinalScore.class)})
    public Object finalScoreEdit(@ApiParam(value = "主键", required = true) @RequestParam Long id) throws IOException {
        TCFinalScore tcFinalScore = tcFinalScoreService.getById(id);
        Objects.requireNonNull(tcFinalScore, "未找到期末成绩信息");
        return ResultUtil.ok(tcFinalScoreService.getById(id));
    }

    @ApiOperation(value = "期末成绩保存")
    @RequestMapping(value = "/final_score/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存", response = EditResult.class)})
    @Transactional
    public Object finalScoreSave(@ApiParam(value = "期末考试成绩信息", required = true) @Valid @RequestBody TCFinalScore tcFinalScore) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩启用/禁用")
    @RequestMapping(value = "/final_score/enable", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    @Transactional
    public Object finalScoreEnable(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                   @ApiParam(value = "启用/禁用", required = true) @RequestParam Boolean enable) throws IOException {
        TCFinalScore tcFinalScore = tcFinalScoreService.getById(id);
        Objects.requireNonNull(tcFinalScore, "未找到期末成绩信息");
        tcFinalScore.setEnable(enable);
        return ResultUtil.ok(tcFinalScoreService.updateById(tcFinalScore));
    }

    @ApiOperation(value = "期末成绩试卷蓝图保存")
    @RequestMapping(value = "/final_score/paper_struct/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "试卷蓝图保存", response = Object.class)})
    @Transactional
    public Object finalScorePaperStructSave(@ApiParam(value = "试卷蓝图结构", required = true) @RequestBody Object body) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩试卷蓝图查询")
    @RequestMapping(value = "/final_score/paper_struct/query", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "试卷蓝图保存", response = Object.class)})
    public Object finalScorePaperStructQuery(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                             @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        return ResultUtil.ok(true);
    }
}
